//**********************************************************************************************************
// author	Tony Madden	(tm@mqware.com)
//**********************************************************************************************************
#include	<maxconnworkset.hpp>
#include	<x_exception.hpp>
#include	<x_mq.hpp>

//**********************************************************************************************************
//**********************************************************************************************************
//**********************************************************************************************************
//
//  MaxConnWorkSet class
//
//
//**********************************************************************************************************
//**********************************************************************************************************
//**********************************************************************************************************

MaxConnWorkSet::MaxConnWorkSet(){
	initialize();

}

//**********************************************************************************************************
// Re-initialize object
//**********************************************************************************************************
void	MaxConnWorkSet::reset(){
	if (pQueue){
		pQueue->close();
		if (pQueue->reasonCode() != MQRC_NONE){
			ostringstream	ost;
			ost << "Error -  MQCLOSE for queue " << sQName << ". Reason: " <<  X_Mq::convertToStringMQRC(pQueue->reasonCode()) ; 
			throw X_Exception(E_ERROR,0,ost.str());
		}
	}
	initialize();
}

//**********************************************************************************************************
// Initialize object
//**********************************************************************************************************
void	MaxConnWorkSet::initialize(){

	memset(bootstrapCorrelationId,0,MQ_CORREL_ID_LENGTH);
	memset(volatileCorrelationId,0,MQ_CORREL_ID_LENGTH);

	lOpenType = 0;

	pQueue = 0;

	bClearAllMessages = false;

}

//**********************************************************************************************************
// Get the correlation id used by Bootstrap message
//**********************************************************************************************************
const char* MaxConnWorkSet::getBootstrapCorrelationId(){
	return (char*)bootstrapCorrelationId;
}

//**********************************************************************************************************
// Get the correlation id used by Volatile message
//**********************************************************************************************************
const char* MaxConnWorkSet::getVolatileCorrelationId(){
	return (char*)volatileCorrelationId;
}

//**********************************************************************************************************
// Get the open type
//**********************************************************************************************************
long	MaxConnWorkSet::getOpenType(){
	return lOpenType;
}

//**********************************************************************************************************
// Get the qname
//**********************************************************************************************************
string	MaxConnWorkSet::getQName(){
	return sQName;
}

//**********************************************************************************************************
// Get the svrconn channel name
//**********************************************************************************************************
string	MaxConnWorkSet::getChannelName(){
	return sChannelName;
}

//**********************************************************************************************************
// Get the ImqQueue object
//**********************************************************************************************************
ImqQueue* 	MaxConnWorkSet::getQueue(){
	if (!pQueue){
		pQueue = new ImqQueue();
	}
	return pQueue;
}

//**********************************************************************************************************
// Set the correlation id for bootstrap messages
//**********************************************************************************************************
void MaxConnWorkSet::setBootstrapCorrelationId(char	arg_acId[],long	arg_lLength){
	if (arg_lLength > MQ_CORREL_ID_LENGTH){
		memcpy(bootstrapCorrelationId,arg_acId,MQ_CORREL_ID_LENGTH);
	}else{
		memcpy(bootstrapCorrelationId,arg_acId,arg_lLength);
	}
}

//**********************************************************************************************************
// Set the correlation id for Volatile messages
//**********************************************************************************************************
void MaxConnWorkSet::setVolatileCorrelationId(char	arg_acId[],long	arg_lLength){
	if (arg_lLength > MQ_CORREL_ID_LENGTH){
		memcpy(volatileCorrelationId,arg_acId,MQ_CORREL_ID_LENGTH);
	}else{
		memcpy(volatileCorrelationId,arg_acId,arg_lLength);
	}
}

//**********************************************************************************************************
// Set the open type
//**********************************************************************************************************
void	MaxConnWorkSet::setOpenType(long arg_l){
	lOpenType = arg_l;
}

//**********************************************************************************************************
// Set the queue name
//**********************************************************************************************************
void	MaxConnWorkSet::setQName(string arg_sQ){
	sQName = arg_sQ;
}

//**********************************************************************************************************
// Set the svrconn channel name
//**********************************************************************************************************
void	MaxConnWorkSet::setChannelName(string arg_sC){
	sChannelName = arg_sC;
}



//**********************************************************************************************************
//**********************************************************************************************************
//**********************************************************************************************************
//
//  MaxConnStateData class
//
// Encapsulates State data
//
//**********************************************************************************************************
//**********************************************************************************************************
//**********************************************************************************************************

//**********************************************************************************************************
// Construct
//**********************************************************************************************************
MaxConnStateData::MaxConnStateData(){
	initialize();
}

//**********************************************************************************************************
// Initialize
//**********************************************************************************************************
void	MaxConnStateData::initialize(){
	memset((void*)&sd,0,sizeof(MXC_STATEDATA));
}

//**********************************************************************************************************
// Re-initialize
//**********************************************************************************************************
void	MaxConnStateData::reset(){
	initialize();
}

//**********************************************************************************************************
// Get event flag
//**********************************************************************************************************
long	MaxConnStateData::getEventFlag(){
	return sd.lEventFlag;
}

//**********************************************************************************************************
// Get event queue name
//**********************************************************************************************************
string	MaxConnStateData::getEventQueue(){
	return string(sd.szEventQueue);
}

//**********************************************************************************************************
// Get event state
//**********************************************************************************************************
long	MaxConnStateData::getEventState(){
	return sd.lEventState;
}

//**********************************************************************************************************
// Get number of existing connections
//**********************************************************************************************************
long	MaxConnStateData::getExistingConnections(){
	return sd.lExistingConnections;
}

//**********************************************************************************************************
// Get frequency of PCF checks
//**********************************************************************************************************
long	MaxConnStateData::getFrequencyOfPCFChecks(){
	return sd.lFrequencyOfPCFChecks;
}

//**********************************************************************************************************
// Get log file name
//**********************************************************************************************************
string	MaxConnStateData::getLogFileName(){
	return string(sd.szLogFileName);
}

//**********************************************************************************************************
// Get log file directory
//**********************************************************************************************************
string	MaxConnStateData::getLogFileDirectory(){
	return string(sd.szLogFileDirectory);
}

//**********************************************************************************************************
// Get max connections value
//**********************************************************************************************************
long	MaxConnStateData::getMaxConnections(){
	return sd.lMaxConnections;
}

//**********************************************************************************************************
// Get max number of in use attempts
//**********************************************************************************************************
long	MaxConnStateData::getMaxNumberOfInUseAttempts(){
	return sd.lMaxNumberOfInUseAttempts;
}

//**********************************************************************************************************
//  Convert the class STATEDATA variable into a delimited string 
//
// arg_pcString					the delimited string
//**********************************************************************************************************
void	MaxConnStateData::getStateDataAsString(char*	arg_pcString){
	// Use the STATEDATA C structure to create the text string which will be written to the State queue (bootstrap message).
	// This is a C structure because the createStateDataString() function is a C function shared with the maxconn module (written in C) 
	// The StateData class encapsulates a STATEDATA variable.
	createStateDataString(sd,arg_pcString,MXC_BOOTSTRAP_PREFIX);
}

//**********************************************************************************************************
// Load the class STATEDATA variable from a delimited array 
//
// arg_pcData				the delimited array
// arg_lLength				length of the array
//**********************************************************************************************************
void	MaxConnStateData::loadStateData(char*	arg_pcData,long	arg_lLength){

	reset();
	
	char	szError[1000];
	// Populate the STATEDATA C structure.
	// This is a C structure because the extractFromStateDataString() function is a C function shared with the maxconn module (written in C) 
	// The StateData class encapsulates a STATEDATA variable.
	if (extractFromStateDataString(&sd ,arg_pcData,arg_lLength,szError)){
		ostringstream	ost;
		ost << "Error - Error parsing state data: "  << szError;
		throw X_Exception(E_ERROR,0,ost.str());
	}
}

//**********************************************************************************************************
// Get state data type (bootstrap or volatile)
//**********************************************************************************************************
string	MaxConnStateData::getStateDataType(){
	// Convert char into string...
	char	a[2];
	a[0] = sd.cStateDataType;
	a[1] = 0;
	return string(a);
}

//**********************************************************************************************************
// Get the number of times the exit reported max connections since the last PCF check
//**********************************************************************************************************
long	MaxConnStateData::getTimesAtConnMaxSinceLastPCFCheck(){
	return sd.lTimesAtConnMaxSinceLastPCFCheck;
}

//**********************************************************************************************************
// Get trace level
//**********************************************************************************************************
long	MaxConnStateData::getTraceLevel(){
	return sd.lTraceLevel;
}

//**********************************************************************************************************
// Get the warn threshold number of connections 
//**********************************************************************************************************
long	MaxConnStateData::getWarnThreshold(){
	return sd.lWarnThreshold;
}

//**********************************************************************************************************
// Set event flag
//**********************************************************************************************************
void	MaxConnStateData::setEventFlag(long arg_l){
	sd.lEventFlag = arg_l;
}

//**********************************************************************************************************
// Set the event queue name
//**********************************************************************************************************
void	MaxConnStateData::setEventQueue(string arg_s){
	string sMethodName = "MaxConnStateData::setEventQueue";

	if (arg_s.size() > sizeof(sd.szLogFileDirectory)-1){
		ostringstream	ost;
		ost << "Error - eventQueue value is "<< arg_s.size() << " characters long. Maximum allowed is " << MQ_Q_NAME_LENGTH;
		throw X_Exception(E_ERROR,0,ost.str());
	}
	strcpy(sd.szEventQueue,arg_s.c_str());
}

//**********************************************************************************************************
// Set event state
//**********************************************************************************************************
void	MaxConnStateData::setEventState(long arg_l){
	sd.lEventState = arg_l;
}

//**********************************************************************************************************
// Set the number of existing connections
//**********************************************************************************************************
void	MaxConnStateData::setExistingConnections(long arg_l){
	sd.lExistingConnections = arg_l;
}

//**********************************************************************************************************
// Set the frequency of PCF checks
//**********************************************************************************************************
void	MaxConnStateData::setFrequencyOfPCFChecks(long arg_l){
	sd.lFrequencyOfPCFChecks = arg_l;
}

//**********************************************************************************************************
// Set the log file directory
//**********************************************************************************************************
void	MaxConnStateData::setLogFileDirectory(string arg_s){
	string sMethodName = "MaxConnStateData::setLogFileDirectory";
	
	if (arg_s.size() > sizeof(sd.szLogFileDirectory)-1){
		ostringstream	ost;
		ost << sMethodName << ". Error - logFileDirectory value is " << arg_s.size() << " characters long. Maximum allowed is " << sizeof(sd.szLogFileDirectory)-1;
		throw X_Exception(E_ERROR,0,ost.str());
	}

	strcpy(sd.szLogFileDirectory,arg_s.c_str());
}

//**********************************************************************************************************
// Set the log file name
//**********************************************************************************************************
void	MaxConnStateData::setLogFileName(string arg_s){
	string sMethodName = "MaxConnStateData::setLogFileName";

	if (arg_s.size() > sizeof(sd.szLogFileDirectory)-1){
		ostringstream	ost;
		ost << sMethodName << ". Error - logFileDirectory value is " << arg_s.size() << " characters long. Maximum allowed is " << sizeof(sd.szLogFileName)-1;
		throw X_Exception(E_ERROR,0,ost.str());
	}
	strcpy(sd.szLogFileName,arg_s.c_str());
}

//**********************************************************************************************************
// Set the max connections
//**********************************************************************************************************
void	MaxConnStateData::setMaxConnections(long arg_l){
	sd.lMaxConnections = arg_l;
}

//**********************************************************************************************************
// Set the max number of in use attempts
//**********************************************************************************************************
void	MaxConnStateData::setMaxNumberOfInUseAttempts(long arg_l){
	sd.lMaxNumberOfInUseAttempts = arg_l;
}

//**********************************************************************************************************
// Set state data type (bootstrap or volatile)
//**********************************************************************************************************
void	MaxConnStateData::setStateDataType(string arg_s){
	if (arg_s.size()){
		sd.cStateDataType = arg_s[0];
	};
}

//**********************************************************************************************************
// Set times at max connections since last PCF check
//**********************************************************************************************************
void	MaxConnStateData::setTimesAtConnMaxSinceLastPCFCheck(long arg_l){
	sd.lTimesAtConnMaxSinceLastPCFCheck = arg_l;
}

//**********************************************************************************************************
// Set trace level
//**********************************************************************************************************
void	MaxConnStateData::setTraceLevel(long arg_l){
	sd.lTraceLevel = arg_l;
}

//**********************************************************************************************************
// Set warn threshold
//**********************************************************************************************************
void	MaxConnStateData::setWarnThreshold(long arg_l){
	sd.lWarnThreshold = arg_l;
}

