//***********************************************************************************************************
//
// The X_String class has a load of string manipulation functions
//
//
// author	Tony Madden	(tm@mqware.com)
//***********************************************************************************************************
#include	<x_string.hpp>


//********************************************************************
// Check for equality ignoring case
//
// return		true if equal (ignoring case)
//********************************************************************
bool X_String::equalsIgnoreCase(const string& arg_sOne,const string& arg_sTwo){

	bool	bRc;

	if (arg_sOne == arg_sTwo){
		bRc = true;
	}else{
		if (toLower(arg_sOne) == toLower(arg_sTwo)){
			bRc = true;
		}else{
			bRc = false;
		}
	}

	return bRc;
}
//********************************************************************
// Convert a string to upper case
//********************************************************************
string X_String::toUpper(const string& arg_s){

	string	sUpper;

	for (int i = 0;i < arg_s.size(); i++){
		sUpper += toupper(arg_s[i]);
	}

	return sUpper;
}
 //********************************************************************
 // Convert a string to lower case
 //********************************************************************
string X_String::toLower(const string& arg_s){

	string	sLower;

	for (int i = 0;i < arg_s.size(); i++){
		sLower += tolower(arg_s[i]);
	}

	return sLower;
}
//********************************************************************
// Check whether string is numeric
//
// return			true if numeric
//********************************************************************
bool	X_String::isNumeric(const  string& arg_s){
	int	i;
	bool	bRc;

	if (arg_s.size()){
		bRc = true;
		if (arg_s[0] == '-' || arg_s[0] == '+'){
			i = 1;
		}else{
			i = 0;
		}
		for (; i < arg_s.size();i++){
			if (!isdigit(arg_s[i])){
				bRc = false;
				break;
			}
		}
	}else{
		bRc = false;
	}
	return bRc;
}

//********************************************************************
// Check whether string is alphabetic
//
// return true if alphabetic
//********************************************************************
bool	X_String::isAlpha(const string& arg_s){
	bool	bRc = true;

	for (int i = 0; i < arg_s.size();i++){
		if (!isalpha((arg_s)[i])){
			bRc = false;
			break;
		}
	}
	return bRc;
}

//********************************************************************
// Check whether string is alphanumeric
//
// return			true if alphanumeric
//********************************************************************
bool	X_String::isAlphaNumeric(const string& arg_s){
	bool	bRc = true;

	for (int i = 0; i < arg_s.size();i++){
		if (!isalpha((arg_s)[i])){
			bRc =  false;
			break;
		}
		if (!isdigit((arg_s)[i])){
			bRc = false;
			break;
		}
	}
	return bRc;
}

//********************************************************************
// Check whether string is displayable
//
// return			true if displayable
//********************************************************************
bool	X_String::isPrint(const string& arg_s){
	bool	bRc = true;
	for (int i = 0; i < arg_s.size();i++){
		if (!isprint((arg_s)[i])){
			bRc = false;
			break;
		}
	}
	return bRc;
}

//********************************************************************
 // Convert string to number
 //********************************************************************
long	X_String::stringToLong(const string& arg_s){
	string sMethodName = "X_String::stringToLong";

	if (!isNumeric(arg_s)){
		ostringstream	ost;
		ost << "Error: string value <" << arg_s <<"> is not numeric";
		throw X_Exception (E_ERROR,0,ost.str(),sMethodName);
	}
	return atol(arg_s.c_str());

}

//********************************************************************
// Remove an escape character
//********************************************************************
string	X_String::removeEscapeCharacter(string arg_s, char	arg_c){

	bool	bPreviousWasEscape = false;
	string	sOut;
	
	for (int i = 0; i < arg_s.size(); i++){
		if (arg_s[i] == arg_c && !bPreviousWasEscape){
			bPreviousWasEscape = true;
		}else{
			if (bPreviousWasEscape){
				bPreviousWasEscape = false;
			}
			sOut += arg_s[i];
		}
	}

	return sOut;

}

//********************************************************************
// Remove leading and trailing spaces
//********************************************************************
string	X_String::trim(const string& arg_s){
	return trim(arg_s,0);
}

//********************************************************************
// Remove leading and trailing spaces and truncate if necessary
//
// arg_s			input string
// arg_lMaxLength	the max length for the value
//
// return			the processed value
//********************************************************************
string	X_String::trim(const string& arg_s,long	arg_lMaxLength){
	string	sOut;
	string sIn = arg_s;

	if (arg_s.size()){
		if (arg_lMaxLength && (arg_s.size() > arg_lMaxLength)){
			sIn = sIn.substr(0,arg_lMaxLength);
		}

		string::size_type st = sIn.find_first_not_of(' ',0);
		if (st == string::npos){
			sOut = "";
		}else{
			sOut = sIn.substr(st);
			st = sOut.find_last_not_of(' ',sOut.size()-1);
			if (st != string::npos && st < sOut.size()-1){
				sOut = sOut.substr(0,st+1);
			}
		}
	}else{
		sOut = "";
	}
			
	return sOut;
 }

//********************************************************************
// Prefix pad characters to data up to specified size
//
// arg_s				input string
// arg_cPadByte			the pad character
// arg_lTotalLength		total length after padding
//********************************************************************
string	X_String::padBefore(const string& arg_s,char arg_cPadByte,long	arg_lTotalLength){
	return pad(arg_s,arg_cPadByte,arg_lTotalLength,false);
}

//********************************************************************
// Suffix pad characters to data up to specified size
//
// arg_s				input string
// arg_cPadByte			the pad character
// arg_lTotalLength		total length after padding
//********************************************************************
string	X_String::padAfter(const string&  arg_s,char arg_cPadByte,long	arg_lTotalLength){
	return pad(arg_s,arg_cPadByte,arg_lTotalLength,true);
}

//********************************************************************
// Pad up to specified size
//
// arg_s				input string
// arg_cPadByte			the pad character
// arg_lTotalLength		total length after padding
// arg_bPostPad			true to suffix data, false to prefix data with pad characters
//********************************************************************
string	X_String::pad(const string& arg_s,char arg_cPadByte,long	arg_lTotalLength,bool arg_bPostPad){
	string sMethodName = "X_String::pad";
	string sOut;

	if (arg_s.size() < arg_lTotalLength){

		long	lPadLength = arg_lTotalLength - arg_s.size();

		char* pcPadder = new char[lPadLength + 1];

		*(pcPadder + lPadLength) = 0;

		memset(pcPadder,arg_cPadByte,lPadLength);

		if (arg_bPostPad ){
			sOut = arg_s + pcPadder;
		}else{
			sOut = pcPadder + arg_s;
		}

		delete[] pcPadder;
	}else{
		sOut = arg_s;
	}
	return sOut;
}




string	X_String::getVersion(){
	return X_STRING_VERSION;
}

