//***********************************************************************************************************
//
// The X_Tokenizer class takes in a delimited string and parses it into tokens
//
//
// author	Tony Madden	(tm@mqware.com)
//***********************************************************************************************************

#ifndef X_TOKENIZER_DEFINED
#define X_TOKENIZER_DEFINED

#define X_TOKENIZER_VERSION		"1.0.0"

#include	<vector>
#include	<string>
#include	<cstdio>

#include	<x_exception.hpp>
#include	<x_string.hpp>

#define		TK_DEFAULT_ESCAPE_CHARACTER		'\\'

using namespace std;

class	X_Tokenizer {
private:
	bool	bTrimTokens;
	bool	bProcessEscapeCharacters;
	long	lCurrentIndex;
	char	cEscapeCharacter;
	string	sOriginalString;

	vector<string>	tokenVector;

	void			enclosedByBothTokens(const string&,const string&,const string&);
	void			initialize();
	void			reset();
	void			tokenParser(const string&,char,bool);
	
public:
	X_Tokenizer();

	virtual ~X_Tokenizer(){};

	const vector<string>*	getAllTokens();
	void					getAllTokens(vector<string>*);
	char					getEscapeCharacter();
	bool					getToken(long,string*);
	bool					getFirstToken(string*);
	string					getToken(long);
	string					getFirstToken();
	string					getNextToken();
	bool					getNextToken(string*);
	string					getOriginalString();
	static string			getVersion();
	bool					isProcessEscapeCharacters();
	long					numberOfTokens();
	void					parse(const string&,string,string);
	void					parse(const string& ,char);
	void					parse(const string&);
	bool					setCurrentIndex(long);
	void					setEscapeCharacter(char);
	void					processEscapeCharacters(bool);
	void					trimTokens(bool);
	bool					isTrimTokens();
	
};




#endif

